@@ -81,6 +81,9 @@ static FILE * fp_fastapairs = 0;
 static FILE * fp_matched = 0;
 static FILE * fp_notmatched = 0;
 
+static int count_matched = 0;
+static int count_notmatched = 0;
+
 inline int allpairs_hit_compare_typed(struct hit * x, struct hit * y)
 {
   // high id, then low id
@@ -213,23 +216,31 @@ void allpairs_output_results(int hit_count,
 
   if (hit_count)
     {
+      count_matched++;
       if (opt_matched)
-        {
-          fasta_print(fp_matched,
-                      query_head,
-                      qsequence,
-                      qseqlen);
-        }
+        fasta_print_general(fp_matched,
+                            0,
+                            qsequence,
+                            qseqlen,
+                            query_head,
+                            strlen(query_head),
+                            0,
+                            count_matched,
+                            -1, -1, 0, 0.0);
     }
   else
     {
+      count_notmatched++;
       if (opt_notmatched)
-        {
-          fasta_print(fp_notmatched,
-                      query_head,
-                      qsequence,
-                      qseqlen);
-        }
+        fasta_print_general(fp_notmatched,
+                            0,
+                            qsequence,
+                            qseqlen,
+                            query_head,
+                            strlen(query_head),
+                            0,
+                            count_notmatched,
+                            -1, -1, 0, 0.0);
     }
 }
 
@@ -534,7 +545,7 @@ void allpairs_global(char * cmdline, char * progheader)
 
   if (opt_alnout)
     {
-      fp_alnout = fopen(opt_alnout, "w");
+      fp_alnout = fopen_output(opt_alnout);
       if (! fp_alnout)
         fatal("Unable to open alignment output file for writing");
 
@@ -544,49 +555,49 @@ void allpairs_global(char * cmdline, char * progheader)
 
   if (opt_samout)
     {
-      fp_samout = fopen(opt_samout, "w");
+      fp_samout = fopen_output(opt_samout);
       if (! fp_samout)
         fatal("Unable to open SAM output file for writing");
     }
 
   if (opt_userout)
     {
-      fp_userout = fopen(opt_userout, "w");
+      fp_userout = fopen_output(opt_userout);
       if (! fp_userout)
         fatal("Unable to open user-defined output file for writing");
     }
 
   if (opt_blast6out)
     {
-      fp_blast6out = fopen(opt_blast6out, "w");
+      fp_blast6out = fopen_output(opt_blast6out);
       if (! fp_blast6out)
         fatal("Unable to open blast6-like output file for writing");
     }
 
   if (opt_uc)
     {
-      fp_uc = fopen(opt_uc, "w");
+      fp_uc = fopen_output(opt_uc);
       if (! fp_uc)
         fatal("Unable to open uc output file for writing");
     }
 
   if (opt_fastapairs)
     {
-      fp_fastapairs = fopen(opt_fastapairs, "w");
+      fp_fastapairs = fopen_output(opt_fastapairs);
       if (! fp_fastapairs)
         fatal("Unable to open fastapairs output file for writing");
     }
 
   if (opt_matched)
     {
-      fp_matched = fopen(opt_matched, "w");
+      fp_matched = fopen_output(opt_matched);
       if (! fp_matched)
         fatal("Unable to open matched output file for writing");
     }
 
   if (opt_notmatched)
     {
-      fp_notmatched = fopen(opt_notmatched, "w");
+      fp_notmatched = fopen_output(opt_notmatched);
       if (! fp_notmatched)
         fatal("Unable to open notmatched output file for writing");
     }
