@@ -785,14 +785,20 @@ int eval_parents(struct chimera_info_s * ci)
       int sumL = best_left_n + best_left_a + best_left_y;
       int sumR = best_right_n + best_right_a + best_right_y;
 
-      if (best_h >= opt_minh)
+      if (opt_uchime2_denovo || opt_uchime3_denovo)
         {
-          status = 3;
-          if ((divdiff >= opt_mindiv) &&
-              (sumL >= opt_mindiffs) &&
-              (sumR >= opt_mindiffs))
+          if ((QM == 100.0) && (QT < 100.0))
             status = 4;
         }
+      else
+        if (best_h >= opt_minh)
+          {
+            status = 3;
+            if ((divdiff >= opt_mindiv) &&
+                (sumL >= opt_mindiffs) &&
+                (sumR >= opt_mindiffs))
+              status = 4;
+          }
 
       /* print alignment */
 
@@ -803,15 +809,33 @@ int eval_parents(struct chimera_info_s * ci)
           fprintf(fp_uchimealns, "\n");
           fprintf(fp_uchimealns, "----------------------------------------"
                   "--------------------------------\n");
-          fprintf(fp_uchimealns, "Query   (%5d nt) %s\n",
-                  ci->query_len, ci->query_head);
-          fprintf(fp_uchimealns, "ParentA (%5" PRIu64 " nt) %s\n",
-                  db_getsequencelen(seqno_a),
-                  db_getheader(seqno_a));
-          fprintf(fp_uchimealns, "ParentB (%5" PRIu64 " nt) %s\n",
-                  db_getsequencelen(seqno_b),
-                  db_getheader(seqno_b));
-          fprintf(fp_uchimealns, "\n");
+          fprintf(fp_uchimealns, "Query   (%5d nt) ",
+                  ci->query_len);
+          if (opt_xsize)
+            abundance_fprint_header_strip_size(fp_uchimealns,
+                                               ci->query_head,
+                                               ci->query_head_len);
+          else
+            fprintf(fp_uchimealns, "%s", ci->query_head);
+
+          fprintf(fp_uchimealns, "\nParentA (%5" PRIu64 " nt) ",
+                  db_getsequencelen(seqno_a));
+          if (opt_xsize)
+            abundance_fprint_header_strip_size(fp_uchimealns,
+                                               db_getheader(seqno_a),
+                                               db_getheaderlen(seqno_a));
+          else
+            fprintf(fp_uchimealns, "%s", db_getheader(seqno_a));
+
+          fprintf(fp_uchimealns, "\nParentB (%5" PRIu64 " nt) ",
+                  db_getsequencelen(seqno_b));
+          if (opt_xsize)
+            abundance_fprint_header_strip_size(fp_uchimealns,
+                                               db_getheader(seqno_b),
+                                               db_getheaderlen(seqno_b));
+          else
+            fprintf(fp_uchimealns, "%s", db_getheader(seqno_b));
+          fprintf(fp_uchimealns, "\n\n");
 
           int width = opt_alignwidth > 0 ? opt_alignwidth : alnlen;
           qpos = 0;
@@ -883,53 +907,71 @@ int eval_parents(struct chimera_info_s * ci)
 
       if (opt_uchimeout)
         {
-          if (opt_uchimeout5)
+          fprintf(fp_uchimeout, "%.4f\t", best_h);
+
+          if (opt_xsize)
             {
-              fprintf(fp_uchimeout,
-                      "%.4f\t%s\t%s\t%s\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t"
-                      "%d\t%d\t%d\t%d\t%d\t%d\t%.1f\t%c\n",
-                      best_h,
-                      ci->query_head,
-                      db_getheader(seqno_a),
-                      db_getheader(seqno_b),
-                      QM,
-                      QA,
-                      QB,
-                      AB,
-                      QT,
-                      best_left_y,
-                      best_left_n,
-                      best_left_a,
-                      best_right_y,
-                      best_right_n,
-                      best_right_a,
-                      divdiff,
-                      status == 4 ? 'Y' : (status == 2 ? 'N' : '?'));
+              abundance_fprint_header_strip_size(fp_uchimeout,
+                                                 ci->query_head,
+                                                 ci->query_head_len);
+              fprintf(fp_uchimeout, "\t");
+              abundance_fprint_header_strip_size(fp_uchimeout,
+                                                 db_getheader(seqno_a),
+                                                 db_getheaderlen(seqno_a));
+              fprintf(fp_uchimeout, "\t");
+              abundance_fprint_header_strip_size(fp_uchimeout,
+                                                 db_getheader(seqno_b),
+                                                 db_getheaderlen(seqno_b));
+              fprintf(fp_uchimeout, "\t");
             }
           else
             {
               fprintf(fp_uchimeout,
-                      "%.4f\t%s\t%s\t%s\t%s\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t"
-                      "%d\t%d\t%d\t%d\t%d\t%d\t%.1f\t%c\n",
-                      best_h,
+                      "%s\t%s\t%s\t",
                       ci->query_head,
                       db_getheader(seqno_a),
-                      db_getheader(seqno_b),
-                      db_getheader(QA >= QB ? seqno_a : seqno_b),
-                      QM,
-                      QA,
-                      QB,
-                      AB,
-                      QT,
-                      best_left_y,
-                      best_left_n,
-                      best_left_a,
-                      best_right_y,
-                      best_right_n,
-                      best_right_a,
-                      divdiff,
-                      status == 4 ? 'Y' : (status == 2 ? 'N' : '?'));
+                      db_getheader(seqno_b));
             }
+
+          if(! opt_uchimeout5)
+            {
+              if (opt_xsize)
+                {
+                  if (QA >= QB)
+                    abundance_fprint_header_strip_size(fp_uchimeout,
+                                                       db_getheader(seqno_a),
+                                                       db_getheaderlen(seqno_a));
+                  else
+                    abundance_fprint_header_strip_size(fp_uchimeout,
+                                                       db_getheader(seqno_b),
+                                                       db_getheaderlen(seqno_b));
+                  fprintf(fp_uchimeout, "\t");
+                }
+              else
+                {
+                  if (QA >= QB)
+                    fprintf(fp_uchimeout, "%s\t", db_getheader(seqno_a));
+                  else
+                    fprintf(fp_uchimeout, "%s\t", db_getheader(seqno_b));
+                }
+            }
+
+          fprintf(fp_uchimeout,
+                  "%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t"
+                  "%d\t%d\t%d\t%d\t%d\t%d\t%.1f\t%c\n",
+                  QM,
+                  QA,
+                  QB,
+                  AB,
+                  QT,
+                  best_left_y,
+                  best_left_n,
+                  best_left_a,
+                  best_right_y,
+                  best_right_n,
+                  best_right_a,
+                  divdiff,
+                  status == 4 ? 'Y' : (status == 2 ? 'N' : '?'));
         }
       pthread_mutex_unlock(&mutex_output);
     }
@@ -942,7 +984,7 @@ int eval_parents(struct chimera_info_s * ci)
   0: no parents, non-chimeric
   1: score < 0 (no alignment), non-chimeric
   2: score < minh, non-chimeric
-  3: score >= minh, suspicious
+  3: score >= minh, suspicious -> not available with uchime2_denovo and uchime3_denovo
   4: score >= minh && (divdiff >= opt_mindiv) && ..., chimeric
 */
 
@@ -1085,45 +1127,6 @@ void chimera_thread_exit(struct chimera_info_s * ci)
     xfree(ci->query_head);
 }
 
-void fasta_print_with_score(FILE * fp,
-                            char * head,
-                            char * seq,
-                            int len,
-                            const char * score_name,
-                            double score)
-{
-  int alloc = strlen(head) + strlen(score_name) + log10(MAX(score,1)) + 16;
-  //  printf("alloc: %d\n", alloc);
-  char * newheader = (char*) xmalloc(alloc);
-  if (head[strlen(head)-1] == ';')
-    snprintf(newheader, alloc, "%s%s=%.4lf;", head, score_name, score);
-  else
-    snprintf(newheader, alloc, "%s%s=%.4lf;", head, score_name, score);
-  fasta_print(fp, newheader, seq, len);
-  xfree(newheader);
-}
-
-void fasta_print_relabel_with_score(FILE * fp,
-                                    char * seq,
-                                    int len,
-                                    char * header,
-                                    int head_len,
-                                    int abundance,
-                                    int ordinal,
-                                    const char * score_name,
-                                    double score)
-{
-  int alloc = head_len + strlen(score_name) + log10(MAX(score,1)) + 16;
-  //printf("head_len: %d   alloc: %d\n", head_len, alloc);
-  char * newheader = (char*) xmalloc(alloc);
-  if (header[head_len-1] == ';')
-    snprintf(newheader, alloc, "%s%s=%.4lf;", header, score_name, score);
-  else
-    snprintf(newheader, alloc, "%s%s=%.4lf;", header, score_name, score);
-  fasta_print_relabel(fp, seq, len, newheader, strlen(newheader), abundance, ordinal);
-  xfree(newheader);
-}
-
 uint64_t chimera_thread_core(struct chimera_info_s * ci)
 {
   chimera_thread_init(ci);
@@ -1282,9 +1285,9 @@ uint64_t chimera_thread_core(struct chimera_info_s * ci)
                 xfree(ci->nwcigar[i]);
                       
               nwcigar = xstrdup(lma.align(ci->query_seq,
-                                         tseq,
-                                         ci->query_len,
-                                         tseqlen));
+                                          tseq,
+                                          ci->query_len,
+                                          tseqlen));
               lma.alignstats(nwcigar,
                              ci->query_seq,
                              tseq,
@@ -1332,41 +1335,42 @@ uint64_t chimera_thread_core(struct chimera_info_s * ci)
           chimera_abundance += ci->query_size;
 
           if (opt_chimeras)
-            {
-              if (opt_fasta_score)
-                fasta_print_with_score(fp_chimeras,
-                                       ci->query_head,
-                                       ci->query_seq,
-                                       ci->query_len,
-                                       opt_uchime_ref ? "uchime_ref" : "uchime_denovo",
-                                       ci->best_h);
-              else
-                fasta_print(fp_chimeras,
-                            ci->query_head,
-                            ci->query_seq,
-                            ci->query_len);
-            }
+            fasta_print_general(fp_chimeras,
+                                0,
+                                ci->query_seq,
+                                ci->query_len,
+                                ci->query_head,
+                                ci->query_head_len,
+                                ci->query_size,
+                                chimera_count,
+                                -1,
+                                -1,
+                                opt_fasta_score ? 
+                                ( opt_uchime_ref ?
+                                  "uchime_ref" : "uchime_denovo" ) : 0,
+                                ci->best_h);
         }
       
       if (status == 3)
         {
           borderline_count++;
           borderline_abundance += ci->query_size;
+
           if (opt_borderline)
-            {
-              if (opt_fasta_score)
-                fasta_print_with_score(fp_borderline,
-                                       ci->query_head,
-                                       ci->query_seq,
-                                       ci->query_len,
-                                       opt_uchime_ref ? "uchime_ref" : "uchime_denovo",
-                                       ci->best_h);
-              else
-                fasta_print(fp_borderline,
-                            ci->query_head,
-                            ci->query_seq,
-                            ci->query_len);
-            }
+            fasta_print_general(fp_borderline,
+                                0,
+                                ci->query_seq,
+                                ci->query_len,
+                                ci->query_head,
+                                ci->query_head_len,
+                                ci->query_size,
+                                borderline_count,
+                                -1,
+                                -1,
+                                opt_fasta_score ? 
+                                ( opt_uchime_ref ?
+                                  "uchime_ref" : "uchime_denovo" ) : 0,
+                                ci->best_h);
         }
 
       if (status < 3)
@@ -1377,44 +1381,42 @@ uint64_t chimera_thread_core(struct chimera_info_s * ci)
           /* output no parents, no chimeras */
           if ((status < 2) && opt_uchimeout)
             {
+              fprintf(fp_uchimeout, "0.0000\t");
+
+              if (opt_xsize)
+                abundance_fprint_header_strip_size(fp_uchimeout,
+                                                   ci->query_head,
+                                                   ci->query_head_len);
+              else
+                fprintf(fp_uchimeout, "%s", ci->query_head);
+
               if (opt_uchimeout5)
                 fprintf(fp_uchimeout,
-                        "0.0000\t%s\t*\t*\t*\t*\t*\t*\t*"
-                        "\t0\t0\t0\t0\t0\t0\t*\tN\n",
-                        ci->query_head);
+                        "\t*\t*\t*\t*\t*\t*\t*\t0\t0\t0\t0\t0\t0\t*\tN\n");
               else
                 fprintf(fp_uchimeout,
-                        "0.0000\t%s\t*\t*\t*\t*\t*\t*\t*\t*"
-                        "\t0\t0\t0\t0\t0\t0\t*\tN\n",
-                        ci->query_head);
+                        "\t*\t*\t*\t*\t*\t*\t*\t*\t0\t0\t0\t0\t0\t0\t*\tN\n");
             }
           
           /* uchime_denovo: add non-chimeras to db */
-          if (opt_uchime_denovo)
+          if (opt_uchime_denovo || opt_uchime2_denovo || opt_uchime3_denovo)
             dbindex_addsequence(seqno, opt_qmask);
 
           if (opt_nonchimeras)
-            {
-              if (opt_fasta_score)
-                fasta_print_relabel_with_score(fp_nonchimeras,
-                                               ci->query_seq,
-                                               ci->query_len,
-                                               ci->query_head,
-                                               ci->query_head_len,
-                                               ci->query_size,
-                                               nonchimera_count,
-                                               opt_uchime_ref ?
-                                               "uchime_ref" : "uchime_denovo",
-                                               ci->best_h);
-              else
-                fasta_print_relabel(fp_nonchimeras,
-                                    ci->query_seq,
-                                    ci->query_len,
-                                    ci->query_head,
-                                    ci->query_head_len,
-                                    ci->query_size,
-                                    nonchimera_count);
-            }
+            fasta_print_general(fp_nonchimeras,
+                                0,
+                                ci->query_seq,
+                                ci->query_len,
+                                ci->query_head,
+                                ci->query_head_len,
+                                ci->query_size,
+                                nonchimera_count,
+                                -1,
+                                -1,
+                                opt_fasta_score ? 
+                                ( opt_uchime_ref ?
+                                  "uchime_ref" : "uchime_denovo" ) : 0,
+                                ci->best_h);
         }
       
       for (int i=0; i < ci->cand_count; i++)
@@ -1477,7 +1479,7 @@ void open_chimera_file(FILE * * f, char * name)
 {
   if (name)
     {
-      *f = fopen(name, "w");
+      *f = fopen_output(name);
       if (!*f)
         fatal("Unable to open file %s for writing", name);
     }
@@ -1507,10 +1509,10 @@ void chimera()
   opt_self = 1;
   opt_selfid = 1;
 
-  if (opt_uchime_denovo)
+  if (opt_uchime_denovo || opt_uchime2_denovo || opt_uchime3_denovo)
     opt_threads = 1;
 
-  if (opt_uchime_denovo)
+  if (opt_uchime_denovo || opt_uchime2_denovo || opt_uchime3_denovo)
     opt_maxsizeratio = 1.0 / opt_abskew;
 
   tophits = opt_maxaccepts + opt_maxrejects;
@@ -1530,6 +1532,8 @@ void chimera()
   pthread_mutex_init(&mutex_input, NULL);
   pthread_mutex_init(&mutex_output, NULL);
 
+  char * denovo_dbname = NULL;
+
   /* prepare queries / database */
   if (opt_uchime_ref)
     {
@@ -1547,7 +1551,15 @@ void chimera()
     }
   else
     {
-      db_read(opt_uchime_denovo, 0);
+      
+      if (opt_uchime_denovo)
+        denovo_dbname = opt_uchime_denovo;
+      else if (opt_uchime2_denovo)
+        denovo_dbname = opt_uchime2_denovo;
+      else // opt_uchime3_denovo
+        denovo_dbname = opt_uchime3_denovo;
+
+      db_read(denovo_dbname, 0);
 
       if (opt_qmask == MASK_DUST)
         dust_all();
@@ -1605,7 +1617,7 @@ void chimera()
       if (opt_uchime_ref)
         fprintf(fp_log, "%s", opt_uchime_ref);
       else
-        fprintf(fp_log, "%s", opt_uchime_denovo);
+        fprintf(fp_log, "%s", denovo_dbname);
       fprintf(fp_log, ": %d/%u chimeras (%.1f%%)\n",
               chimera_count,
               seqno, 
