@@ -74,6 +74,9 @@ typedef struct clusterinfo_s
 static clusterinfo_t * clusterinfo = 0;
 static int clusters = 0;
 
+static int count_matched = 0;
+static int count_notmatched = 0;
+
 static int64_t * cluster_abundance;
 
 static FILE * fp_centroids = 0;
@@ -356,6 +359,7 @@ void cluster_core_results_hit(struct hit * best,
                               char * qsequence_rc,
                               int qsize)
 {
+  count_matched++;
 
   if (opt_otutabout || opt_mothur_shared_out || opt_biomout)
     {
@@ -406,7 +410,15 @@ void cluster_core_results_hit(struct hit * best,
                                qsequence, qseqlen, qsequence_rc);
   
   if (opt_matched)
-    fasta_print(fp_matched, query_head, qsequence, qseqlen);
+    fasta_print_general(fp_matched,
+                        0,
+                        qsequence,
+                        qseqlen,
+                        query_head,
+                        strlen(query_head),
+                        qsize,
+                        count_matched,
+                        -1, -1, 0, 0.0);
 }
 
 void cluster_core_results_nohit(int clusterno,
@@ -416,6 +428,7 @@ void cluster_core_results_nohit(int clusterno,
                                 char * qsequence_rc,
                                 int qsize)
 {
+  count_notmatched++;
 
   if (opt_otutabout || opt_mothur_shared_out || opt_biomout)
     {
@@ -447,7 +460,15 @@ void cluster_core_results_nohit(int clusterno,
     }
   
   if (opt_notmatched)
-    fasta_print(fp_notmatched, query_head, qsequence, qseqlen);
+    fasta_print_general(fp_notmatched,
+                        0,
+                        qsequence,
+                        qseqlen,
+                        query_head,
+                        strlen(query_head),
+                        qsize,
+                        count_notmatched,
+                        -1, -1, 0, 0.0);
 }
 
 int compare_kmersample(const void * a, const void * b)
@@ -989,21 +1010,21 @@ void cluster(char * dbname,
 {
   if (opt_centroids)
     {
-      fp_centroids = fopen(opt_centroids, "w");
+      fp_centroids = fopen_output(opt_centroids);
       if (!fp_centroids)
         fatal("Unable to open centroids file for writing");
     }
 
   if (opt_uc)
     {
-      fp_uc = fopen(opt_uc, "w");
+      fp_uc = fopen_output(opt_uc);
       if (!fp_uc)
         fatal("Unable to open uc file for writing");
     }
 
   if (opt_alnout)
     {
-      fp_alnout = fopen(opt_alnout, "w");
+      fp_alnout = fopen_output(opt_alnout);
       if (! fp_alnout)
         fatal("Unable to open alignment output file for writing");
 
@@ -1013,63 +1034,63 @@ void cluster(char * dbname,
 
   if (opt_samout)
     {
-      fp_samout = fopen(opt_samout, "w");
+      fp_samout = fopen_output(opt_samout);
       if (! fp_samout)
         fatal("Unable to open SAM output file for writing");
     }
 
   if (opt_userout)
     {
-      fp_userout = fopen(opt_userout, "w");
+      fp_userout = fopen_output(opt_userout);
       if (! fp_userout)
         fatal("Unable to open user-defined output file for writing");
     }
 
   if (opt_blast6out)
     {
-      fp_blast6out = fopen(opt_blast6out, "w");
+      fp_blast6out = fopen_output(opt_blast6out);
       if (! fp_blast6out)
         fatal("Unable to open blast6-like output file for writing");
     }
 
   if (opt_fastapairs)
     {
-      fp_fastapairs = fopen(opt_fastapairs, "w");
+      fp_fastapairs = fopen_output(opt_fastapairs);
       if (! fp_fastapairs)
         fatal("Unable to open fastapairs output file for writing");
     }
 
   if (opt_matched)
     {
-      fp_matched = fopen(opt_matched, "w");
+      fp_matched = fopen_output(opt_matched);
       if (! fp_matched)
         fatal("Unable to open matched output file for writing");
     }
 
   if (opt_notmatched)
     {
-      fp_notmatched = fopen(opt_notmatched, "w");
+      fp_notmatched = fopen_output(opt_notmatched);
       if (! fp_notmatched)
         fatal("Unable to open notmatched output file for writing");
     }
 
   if (opt_otutabout)
     {
-      fp_otutabout = fopen(opt_otutabout, "w");
+      fp_otutabout = fopen_output(opt_otutabout);
       if (! fp_otutabout)
         fatal("Unable to open OTU table (text format) output file for writing");
     }
 
   if (opt_mothur_shared_out)
     {
-      fp_mothur_shared_out = fopen(opt_mothur_shared_out, "w");
+      fp_mothur_shared_out = fopen_output(opt_mothur_shared_out);
       if (! fp_mothur_shared_out)
         fatal("Unable to open OTU table (mothur format) output file for writing");
     }
 
   if (opt_biomout)
     {
-      fp_biomout = fopen(opt_biomout, "w");
+      fp_biomout = fopen_output(opt_biomout);
       if (! fp_biomout)
         fatal("Unable to open OTU table (biom 1.0 format) output file for writing");
     }
@@ -1091,9 +1112,9 @@ void cluster(char * dbname,
   
   if (opt_cluster_fast)
     db_sortbylength();
-  else if (opt_cluster_size)
+  else if (opt_cluster_size || opt_cluster_unoise)
     db_sortbyabundance();
-  
+
   dbindex_prepare(1, opt_qmask);
   
   /* tophits = the maximum number of hits we need to store */
@@ -1113,7 +1134,7 @@ void cluster(char * dbname,
 
   if (opt_log)
     {
-      uint64_t slots = 1UL << (opt_wordlength << 1UL);
+      uint64_t slots = 1ULL << (opt_wordlength << 1ULL);
       fprintf(fp_log, "\n");
       fprintf(fp_log, "      Alphabet  nt\n");
       fprintf(fp_log, "    Word width  %" PRId64 "\n", opt_wordlength);
@@ -1188,6 +1209,7 @@ void cluster(char * dbname,
     fn_clusters = (char *) xmalloc(strlen(opt_clusters) + 25);
 
   int lastcluster = -1;
+  int ordinal = 0;
 
   for(int i=0; i<seqcount; i++)
     {
@@ -1201,32 +1223,31 @@ void cluster(char * dbname,
           /* the first sequence is always the centroid */
 
           if (opt_centroids)
-            {
-              fasta_print_relabel(fp_centroids,
-                                  db_getsequence(seqno),
-                                  db_getsequencelen(seqno),
-                                  db_getheader(seqno),
-                                  db_getheaderlen(seqno),
-                                  cluster_abundance[clusterno],
-                                  clusterno+1);
-            }
+            fasta_print_general(fp_centroids,
+                                0,
+                                db_getsequence(seqno),
+                                db_getsequencelen(seqno),
+                                db_getheader(seqno),
+                                db_getheaderlen(seqno),
+                                cluster_abundance[clusterno],
+                                clusterno+1,
+                                -1, -1, 0, 0.0);
 
           if (opt_uc)
-            {
-              fprintf(fp_uc, "C\t%d\t%" PRId64 "\t*\t*\t*\t*\t*\t%s\t*\n",
-                      clusterno,
-                      cluster_abundance[clusterno],
-                      db_getheader(seqno));
-            }
-
+            fprintf(fp_uc, "C\t%d\t%" PRId64 "\t*\t*\t*\t*\t*\t%s\t*\n",
+                    clusterno,
+                    cluster_abundance[clusterno],
+                    db_getheader(seqno));
+          
           if (opt_clusters)
             {
               /* close previous (except for first time) and open new file */
               if (lastcluster != -1)
                 fclose(fp_clusters);
               
+              ordinal = 0;
               sprintf(fn_clusters, "%s%d", opt_clusters, clusterno);
-              fp_clusters = fopen(fn_clusters, "w");
+              fp_clusters = fopen_output(fn_clusters);
               if (!fp_clusters)
                 fatal("Unable to open clusters file for writing");
             }
@@ -1237,7 +1258,10 @@ void cluster(char * dbname,
       /* performed for all sequences */
 
       if (opt_clusters)
-        fasta_print_db(fp_clusters, seqno);
+        {
+          ordinal++;
+          fasta_print_db_relabel(fp_clusters, seqno, ordinal);
+        }
       
       progress_update(i);
     }
@@ -1255,35 +1279,51 @@ void cluster(char * dbname,
 
   progress_done();
 
-  if (!opt_quiet)
+  if (clusters < 1)
     {
-      fprintf(stderr,
-              "Clusters: %d Size min %" PRId64 ", max %" PRId64 ", avg %.1f\n",
-              clusters,
-              abundance_min,
-              abundance_max,
-              1.0 * seqcount / clusters);
-      fprintf(stderr,
-              "Singletons: %d, %.1f%% of seqs, %.1f%% of clusters\n",
-              singletons,
-              100.0 * singletons / seqcount, 
-              100.0 * singletons / clusters);
+      if (!opt_quiet)
+        {
+          fprintf(stderr, "Clusters: 0\n");
+          fprintf(stderr, "Singletons: 0\n");
+        }
+      if (opt_log)
+        {
+          fprintf(fp_log, "Clusters: 0\n");
+          fprintf(fp_log, "Singletons: 0\n");
+        }
     }
-
-  if (opt_log)
+  else
     {
-      fprintf(fp_log,
-              "Clusters: %d Size min %" PRId64 ", max %" PRId64 ", avg %.1f\n",
-              clusters,
-              abundance_min,
-              abundance_max,
-              1.0 * seqcount / clusters);
-      fprintf(fp_log,
-              "Singletons: %d, %.1f%% of seqs, %.1f%% of clusters\n",
-              singletons,
-              100.0 * singletons / seqcount, 
-              100.0 * singletons / clusters);
-      fprintf(fp_log, "\n");
+      if (!opt_quiet)
+        {
+          fprintf(stderr,
+                  "Clusters: %d Size min %" PRId64 ", max %" PRId64 ", avg %.1f\n",
+                  clusters,
+                  abundance_min,
+                  abundance_max,
+                  1.0 * seqcount / clusters);
+          fprintf(stderr,
+                  "Singletons: %d, %.1f%% of seqs, %.1f%% of clusters\n",
+                  singletons,
+                  100.0 * singletons / seqcount,
+                  100.0 * singletons / clusters);
+        }
+
+      if (opt_log)
+        {
+          fprintf(fp_log,
+                  "Clusters: %d Size min %" PRId64 ", max %" PRId64 ", avg %.1f\n",
+                  clusters,
+                  abundance_min,
+                  abundance_max,
+                  1.0 * seqcount / clusters);
+          fprintf(fp_log,
+                  "Singletons: %d, %.1f%% of seqs, %.1f%% of clusters\n",
+                  singletons,
+                  100.0 * singletons / seqcount,
+                  100.0 * singletons / clusters);
+          fprintf(fp_log, "\n");
+        }
     }
 
   if (opt_clusterout_sort)
@@ -1307,15 +1347,15 @@ void cluster(char * dbname,
       FILE * fp_profile = 0;
 
       if (opt_msaout)
-        if (!(fp_msaout = fopen(opt_msaout, "w")))
+        if (!(fp_msaout = fopen_output(opt_msaout)))
           fatal("Unable to open msaout file");
 
       if (opt_consout)
-        if (!(fp_consout = fopen(opt_consout, "w")))
+        if (!(fp_consout = fopen_output(opt_consout)))
           fatal("Unable to open consout file");
 
       if (opt_profile)
-        if (!(fp_profile = fopen(opt_profile, "w")))
+        if (!(fp_profile = fopen_output(opt_profile)))
           fatal("Unable to open profile file");
 
       lastcluster = -1;
@@ -1444,3 +1484,8 @@ void cluster_size(char * cmdline, char * progheader)
 {
   cluster(opt_cluster_size, cmdline, progheader);
 }
+
+void cluster_unoise(char * cmdline, char * progheader)
+{
+  cluster(opt_cluster_unoise, cmdline, progheader);
+}
