@@ -62,16 +62,31 @@
 
 #define MEMCHUNK 16777216
 
-static fastx_handle h;
+static fastx_handle h = 0;
 static bool is_fastq = 0;
 static uint64_t sequences = 0;
 static uint64_t nucleotides = 0;
-static uint64_t longest;
-static uint64_t shortest;
-static uint64_t longestheader;
+static uint64_t longest = 0;
+static uint64_t shortest = 0;
+static uint64_t longestheader = 0;
 
-seqinfo_t * seqindex;
-char * datap;
+seqinfo_t * seqindex = 0;
+char * datap = 0;
+
+void db_setinfo(bool new_is_fastq,
+                uint64_t new_sequences,
+                uint64_t new_nucleotides,
+                uint64_t new_longest,
+                uint64_t new_shortest,
+                uint64_t new_longestheader)
+{
+  is_fastq = new_is_fastq;
+  sequences = new_sequences;
+  nucleotides = new_nucleotides;
+  longest = new_longest;
+  shortest = new_shortest;
+  longestheader = new_longestheader;
+}
 
 bool db_is_fastq()
 {
@@ -88,8 +103,6 @@ char * db_getquality(uint64_t seqno)
 
 void db_read(const char * filename, int upcase)
 {
-  /* compile regexp for abundance pattern */
-
   h = fastx_open(filename);
 
   if (!h)
@@ -113,6 +126,7 @@ void db_read(const char * filename, int upcase)
 
   int64_t discarded_short = 0;
   int64_t discarded_long = 0;
+  int64_t discarded_unoise = 0;
 
   /* allocate space for data */
   uint64_t dataalloc = 0;
@@ -129,9 +143,7 @@ void db_read(const char * filename, int upcase)
     {
       size_t headerlength = fastx_get_header_length(h);
       size_t sequencelength = fastx_get_sequence_length(h);
-
-      unsigned int abundance = abundance_get(global_abundance,
-                                             fastx_get_header(h));
+      int64_t abundance = fastx_get_abundance(h);
       
       if (sequencelength < (size_t)opt_minseqlength)
         {
@@ -141,6 +153,10 @@ void db_read(const char * filename, int upcase)
         {
           discarded_long++;
         }
+      else if (opt_cluster_unoise && (abundance < (int64_t)opt_minsize))
+        {
+          discarded_unoise++;
+        }
       else
         {
           /* grow space for data, if necessary */
@@ -249,27 +265,39 @@ void db_read(const char * filename, int upcase)
   if (discarded_short)
     {
       fprintf(stderr,
-              "WARNING: %" PRId64 " sequences shorter than %" PRId64 " nucleotides discarded.\n",
-              discarded_short, opt_minseqlength);
+              "minseqlength %" PRId64 ": %" PRId64 " %s discarded.\n",
+              opt_minseqlength, discarded_short, (discarded_short == 1 ? "sequence" : "sequences"));
 
       if (opt_log)
         fprintf(fp_log,
-                "WARNING: %" PRId64 " sequences shorter than %" PRId64 " nucleotides discarded.\n\n",
-                discarded_short, opt_minseqlength);
+              "minseqlength %" PRId64 ": %" PRId64 " %s discarded.\n\n",
+              opt_minseqlength, discarded_short, (discarded_short == 1 ? "sequence" : "sequences"));
     }
   
   if (discarded_long)
     {
       fprintf(stderr,
-              "WARNING: %" PRId64 " sequences longer than %" PRId64 " nucleotides discarded.\n",
-              discarded_long, opt_maxseqlength);
+              "maxseqlength %" PRId64 ": %" PRId64 " %s discarded.\n",
+              opt_maxseqlength, discarded_long, (discarded_long == 1 ? "sequence" : "sequences"));
 
       if (opt_log)
         fprintf(fp_log,
-                "WARNING: %" PRId64 " sequences longer than %" PRId64 " nucleotides discarded.\n\n",
-                discarded_long, opt_maxseqlength);
+              "maxseqlength %" PRId64 ": %" PRId64 " %s discarded.\n\n",
+              opt_maxseqlength, discarded_long, (discarded_long == 1 ? "sequence" : "sequences"));
     }
 
+    if (discarded_unoise)
+    {
+      fprintf(stderr,
+              "minsize %" PRId64 ": %" PRId64 " %s discarded.\n",
+              opt_minsize, discarded_unoise, (discarded_unoise == 1 ? "sequence" : "sequences"));
+
+      if (opt_log)
+        fprintf(fp_log,
+              "minsize %" PRId64 ": %" PRId64 " %s discarded.\n",
+              opt_minsize, discarded_unoise, (discarded_unoise == 1 ? "sequence" : "sequences"));
+    }
+            
   show_rusage();
 }
 
