@@ -64,6 +64,9 @@ extern unsigned int * kmerindex; /* the list of matching seqnos for kmers */
 extern bitmap_t * * kmerbitmap;
 extern unsigned int * dbindex_map;
 extern unsigned int dbindex_count;
+extern unsigned int kmerhashsize;
+extern uint64_t kmerindexsize;
+extern uhandle_s * dbindex_uh;
 
 void fprint_kmer(FILE * f, unsigned int k, uint64_t kmer);
 
@@ -71,6 +74,7 @@ void dbindex_prepare(int use_bitmap, int seqmask);
 void dbindex_addallsequences(int seqmask);
 void dbindex_addsequence(unsigned int seqno, int seqmask);
 void dbindex_free();
+void dbindex_udb_write();
 
 inline unsigned char * dbindex_getbitmap(unsigned int kmer)
 {
