@@ -149,14 +149,14 @@ void derep_fulllength()
 
   if (opt_output)
     {
-      fp_output = fopen(opt_output, "w");
+      fp_output = fopen_output(opt_output);
       if (!fp_output)
         fatal("Unable to open output file for writing");
     }
 
   if (opt_uc)
     {
-      fp_uc = fopen(opt_uc, "w");
+      fp_uc = fopen_output(opt_uc);
       if (!fp_uc)
         fatal("Unable to open output (uc) file for writing");
     }
@@ -345,15 +345,26 @@ void derep_fulllength()
   
   average = 1.0 * sumsize / clusters;
 
-  if (!opt_quiet)
-    fprintf(stderr,
-            "%" PRId64 " unique sequences, avg cluster %.1lf, median %.0f, max %" PRIu64 "\n",
-            clusters, average, median, maxsize);
-
-  if (opt_log)
-    fprintf(fp_log,
-            "%" PRId64 " unique sequences, avg cluster %.1lf, median %.0f, max %" PRIu64 "\n\n",
-            clusters, average, median, maxsize);
+  if (clusters < 1)
+    {
+      if (!opt_quiet)
+        fprintf(stderr,
+                "0 unique sequences\n");
+      if (opt_log)
+        fprintf(fp_log,
+                "0 unique sequences\n\n");
+    }
+  else
+    {
+      if (!opt_quiet)
+        fprintf(stderr,
+                "%" PRId64 " unique sequences, avg cluster %.1lf, median %.0f, max %" PRIu64 "\n",
+                clusters, average, median, maxsize);
+      if (opt_log)
+        fprintf(fp_log,
+                "%" PRId64 " unique sequences, avg cluster %.1lf, median %.0f, max %" PRIu64 "\n\n",
+                clusters, average, median, maxsize);
+    }
 
   show_rusage();
   
@@ -388,13 +399,15 @@ void derep_fulllength()
           if ((size >= opt_minuniquesize) && (size <= opt_maxuniquesize))
             {
               relabel_count++;
-              fasta_print_relabel(fp_output,
+              fasta_print_general(fp_output,
+                                  0,
                                   db_getsequence(bp->seqno_first),
                                   db_getsequencelen(bp->seqno_first),
                                   db_getheader(bp->seqno_first),
                                   db_getheaderlen(bp->seqno_first),
                                   size,
-                                  relabel_count);
+                                  relabel_count,
+                                  -1, -1, 0, 0.0);
               if (relabel_count == opt_topn)
                 break;
             }
@@ -475,14 +488,14 @@ void derep_prefix()
 
   if (opt_output)
     {
-      fp_output = fopen(opt_output, "w");
+      fp_output = fopen_output(opt_output);
       if (!fp_output)
         fatal("Unable to open output file for writing");
     }
 
   if (opt_uc)
     {
-      fp_uc = fopen(opt_uc, "w");
+      fp_uc = fopen_output(opt_uc);
       if (!fp_uc)
         fatal("Unable to open output (uc) file for writing");
     }
@@ -566,12 +579,12 @@ void derep_prefix()
 
       /* compute hashes of all prefixes */
 
-      uint64_t fnv1a_hash = 14695981039346656037UL;
+      uint64_t fnv1a_hash = 14695981039346656037ULL;
       prefix_hashes[0] = fnv1a_hash;
       for(unsigned int j = 0; j < seqlen; j++)
         {
           fnv1a_hash ^= seq_up[j];
-          fnv1a_hash *= 1099511628211UL;
+          fnv1a_hash *= 1099511628211ULL;
           prefix_hashes[j+1] = fnv1a_hash;
         }
 
@@ -694,15 +707,26 @@ void derep_prefix()
   
   average = 1.0 * sumsize / clusters;
 
-  if (!opt_quiet)
-    fprintf(stderr,
-            "%" PRId64 " unique sequences, avg cluster %.1lf, median %.0f, max %" PRIu64 "\n",
-            clusters, average, median, maxsize);
-
-  if (opt_log)
-    fprintf(fp_log,
-            "%" PRId64 " unique sequences, avg cluster %.1lf, median %.0f, max %" PRIu64 "\n\n",
-            clusters, average, median, maxsize);
+  if (clusters < 1)
+    {
+      if (!opt_quiet)
+        fprintf(stderr,
+                "0 unique sequences\n");
+      if (opt_log)
+        fprintf(fp_log,
+                "0 unique sequences\n\n");
+    }
+  else
+    {
+      if (!opt_quiet)
+        fprintf(stderr,
+                "%" PRId64 " unique sequences, avg cluster %.1lf, median %.0f, max %" PRIu64 "\n",
+                clusters, average, median, maxsize);
+      if (opt_log)
+        fprintf(fp_log,
+                "%" PRId64 " unique sequences, avg cluster %.1lf, median %.0f, max %" PRIu64 "\n\n",
+                clusters, average, median, maxsize);
+    }
 
   show_rusage();
   
@@ -736,13 +760,15 @@ void derep_prefix()
           if ((size >= opt_minuniquesize) && (size <= opt_maxuniquesize))
             {
               relabel_count++;
-              fasta_print_relabel(fp_output,
+              fasta_print_general(fp_output,
+                                  0,
                                   db_getsequence(bp->seqno_first),
                                   db_getsequencelen(bp->seqno_first),
                                   db_getheader(bp->seqno_first),
                                   db_getheaderlen(bp->seqno_first),
                                   size,
-                                  relabel_count);
+                                  relabel_count,
+                                  -1, -1, 0, 0.0);
               if (relabel_count == opt_topn)
                 break;
             }
