@@ -102,30 +102,30 @@ void filter(bool fastq_only, char * filename)
 
   if (opt_fastaout)
     {
-      fp_fastaout = fopen(opt_fastaout, "w");
+      fp_fastaout = fopen_output(opt_fastaout);
       if (!fp_fastaout)
-        fatal("Unable to open fasta output file for writing");
+        fatal("Unable to open FASTA output file for writing");
     }
 
   if (opt_fastqout)
     {
-      fp_fastqout = fopen(opt_fastqout, "w");
+      fp_fastqout = fopen_output(opt_fastqout);
       if (!fp_fastqout)
-        fatal("Unable to open fastq output file for writing");
+        fatal("Unable to open FASTQ output file for writing");
     }
 
   if (opt_fastaout_discarded)
     {
-      fp_fastaout_discarded = fopen(opt_fastaout_discarded, "w");
+      fp_fastaout_discarded = fopen_output(opt_fastaout_discarded);
       if (!fp_fastaout_discarded)
-        fatal("Unable to open fasta output file for writing");
+        fatal("Unable to open FASTA output file for writing");
     }
 
   if (opt_fastqout_discarded)
     {
-      fp_fastqout_discarded = fopen(opt_fastqout_discarded, "w");
+      fp_fastqout_discarded = fopen_output(opt_fastqout_discarded);
       if (!fp_fastqout_discarded)
-        fatal("Unable to open fastq output file for writing");
+        fatal("Unable to open FASTQ output file for writing");
     }
 
   uint64_t header_alloc = 0;
@@ -142,9 +142,6 @@ void filter(bool fastq_only, char * filename)
   int64_t discarded = 0;
   int64_t truncated = 0;
 
-  char hex_md5[LEN_HEX_DIG_MD5];
-  char hex_sha1[LEN_HEX_DIG_SHA1];
-
   while(fastx_next(h, 0, chrmap_no_change))
     {
       int64_t length = fastx_get_sequence_length(h);
@@ -169,7 +166,16 @@ void filter(bool fastq_only, char * filename)
               length = 0;
             }
         }
-      
+
+      /* strip right end */
+      if (opt_fastq_stripright > 0)
+        {
+          if (opt_fastq_stripright < length)
+            length -= opt_fastq_stripright;
+          else
+            length = 0;
+        }
+
       /* truncate trailing part */
       if (opt_fastq_trunclen >= 0)
         {
@@ -216,7 +222,9 @@ void filter(bool fastq_only, char * filename)
           ((opt_fastq_trunclen < 0) || (length >= opt_fastq_trunclen)) &&
           (ncount <= opt_fastq_maxns) &&
           (ee <= opt_fastq_maxee) &&
-          ((length == 0) || (ee / length <= opt_fastq_maxee_rate)))
+          ((length == 0) || (ee / length <= opt_fastq_maxee_rate)) &&
+          ((opt_minsize == 0) || (abundance >= opt_minsize)) &&
+          ((opt_maxsize == 0) || (abundance <= opt_maxsize)))
         {
           /* keep the sequence */
 
@@ -231,43 +239,30 @@ void filter(bool fastq_only, char * filename)
             }
 
           if (opt_fastaout)
-            {
-              if (opt_eeout || opt_fastq_eeout)
-                fasta_print_relabel_ee(fp_fastaout,
-                                       p, length,
-                                       d, fastx_get_header_length(h),
-                                       abundance, kept,
-                                       ee);
-              else
-                fasta_print_relabel(fp_fastaout,
-                                    p, length,
-                                    d, fastx_get_header_length(h),
-                                    abundance, kept);
-            }
+            fasta_print_general(fp_fastaout,
+                                0,
+                                p,
+                                length,
+                                d,
+                                fastx_get_header_length(h),
+                                abundance,
+                                kept,
+                                -1,
+                                -1,
+                                (opt_eeout || opt_fastq_eeout) ? "ee" : 0,
+                                ee);
+
           if (opt_fastqout)
-            {
-              if (opt_relabel)
-                {
-                  (void) snprintf(header, header_alloc,
-                                  "%s%" PRId64, opt_relabel, kept);
-                  d = header;
-                }
-              else if (opt_relabel_md5)
-                {
-                  get_hex_seq_digest_md5(hex_md5, p, length);
-                  d = hex_md5;
-                }
-              else if (opt_relabel_sha1)
-                {
-                  get_hex_seq_digest_sha1(hex_sha1, p, length);
-                  d = hex_sha1;
-                }
-              
-              if (opt_eeout || opt_fastq_eeout)
-                fastq_print_with_ee(fp_fastqout, d, p, q, ee);
-              else
-                fastq_print(fp_fastqout, d, p, q);
-            }
+            fastq_print_general(fp_fastqout,
+                                p,
+                                length,
+                                d,
+                                fastx_get_header_length(h),
+                                q,
+                                abundance,
+                                kept,
+                                (opt_eeout || opt_fastq_eeout) ? "ee" : 0,
+                                ee);
         }
       else
         {
@@ -275,58 +270,50 @@ void filter(bool fastq_only, char * filename)
 
           discarded++;
 
-          p = fastx_get_sequence(h);
-          q = fastx_get_quality(h);
-
           if (opt_fastaout_discarded)
-            {
-              if (opt_eeout || opt_fastq_eeout)
-                fasta_print_relabel_ee(fp_fastaout_discarded,
-                                       p, length,
-                                       d, fastx_get_header_length(h),
-                                       abundance, discarded,
-                                       ee);
-              else
-                fasta_print_relabel(fp_fastaout_discarded,
-                                    p, length,
-                                    d, fastx_get_header_length(h),
-                                    abundance, discarded);
-            }
+            fasta_print_general(fp_fastaout_discarded,
+                                0,
+                                p,
+                                length,
+                                d,
+                                fastx_get_header_length(h),
+                                abundance,
+                                discarded,
+                                -1,
+                                -1,
+                                (opt_eeout || opt_fastq_eeout) ? "ee" : 0,
+                                ee);
 
           if (opt_fastqout_discarded)
-            {
-              if (opt_relabel)
-                {
-                  (void) snprintf(header, header_alloc, "%s%" PRId64, opt_relabel, discarded);
-                  d = header;
-                }
-              else if (opt_relabel_md5)
-                {
-                  get_hex_seq_digest_md5(hex_md5, p, length);
-                  d = hex_md5;
-                }
-              else if (opt_relabel_sha1)
-                {
-                  get_hex_seq_digest_sha1(hex_sha1, p, length);
-                  d = hex_sha1;
-                }
-
-              if (opt_eeout || opt_fastq_eeout)
-                fastq_print_with_ee(fp_fastqout_discarded, d, p, q, ee);
-              else
-                fastq_print(fp_fastqout_discarded, d, p, q);
-            }
+            fastq_print_general(fp_fastqout_discarded,
+                                p,
+                                length,
+                                d,
+                                fastx_get_header_length(h),
+                                q,
+                                abundance,
+                                discarded,
+                                (opt_eeout || opt_fastq_eeout) ? "ee" : 0,
+                                ee);
         }
 
       progress_update(fastx_get_position(h));
     }
   progress_done();
 
-  fprintf(stderr,
-          "%" PRId64 " sequences kept (of which %" PRId64 " truncated), %" PRId64 " sequences discarded.\n",
-          kept,
-          truncated,
-          discarded);
+  if (! opt_quiet)
+    fprintf(stderr,
+            "%" PRId64 " sequences kept (of which %" PRId64 " truncated), %" PRId64 " sequences discarded.\n",
+            kept,
+            truncated,
+            discarded);
+
+  if (opt_log)
+    fprintf(fp_log,
+            "%" PRId64 " sequences kept (of which %" PRId64 " truncated), %" PRId64 " sequences discarded.\n",
+            kept,
+            truncated,
+            discarded);
 
   if (header)
     xfree(header);
@@ -376,7 +363,7 @@ void fastq_chars()
 
   uint64_t filesize = fastq_get_size(h);
 
-  progress_init("Reading fastq file", filesize);
+  progress_init("Reading FASTQ file", filesize);
 
   uint64_t seq_count = 0;
   
@@ -446,8 +433,6 @@ void fastq_chars()
 
   fastq_close(h);
   
-  fprintf(stderr, "Read %" PRIu64 " sequences.\n", seq_count);
-
   char qmin = 0;
   char qmax = 0;
 
@@ -476,69 +461,142 @@ void fastq_chars()
   else
     fastq_ascii = 64;
 
-  fprintf(stderr, "Qmin %d, QMax %d, Range %d\n",
-          qmin, qmax, qmax-qmin+1);
-
   fastq_qmax = qmax - fastq_ascii;
   fastq_qmin = qmin - fastq_ascii;
-  
-  fprintf(stderr, "Guess: -fastq_qmin %d -fastq_qmax %d -fastq_ascii %d\n",
-          fastq_qmin, fastq_qmax, fastq_ascii);
-  
-  if (fastq_ascii == 64)
-    {
-      if (qmin < 64)
-        fprintf(stderr, "Guess: Solexa format (phred+64)\n");
-      else if (qmin < 66)
-        fprintf(stderr, "Guess: Illumina 1.3+ format (phred+64)\n");
-      else
-        fprintf(stderr, "Guess: Illumina 1.5+ format (phred+64)\n");
-    }
-  else
-    {
-      if (qmax > 73)
-        fprintf(stderr, "Guess: Illumina 1.8+ format (phred+33)\n");
-      else
-        fprintf(stderr, "Guess: Original Sanger format (phred+33)\n");
-    }
 
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Letter          N   Freq MaxRun\n");
-  fprintf(stderr, "------ ---------- ------ ------\n");
-  
-  for(int c=0; c<256; c++)
+  if (!opt_quiet)
     {
-      if (sequence_chars[c] > 0)
+      fprintf(stderr, "Read %" PRIu64 " sequences.\n", seq_count);
+
+      fprintf(stderr, "Qmin %d, QMax %d, Range %d\n",
+              qmin, qmax, qmax-qmin+1);
+
+      fprintf(stderr, "Guess: -fastq_qmin %d -fastq_qmax %d -fastq_ascii %d\n",
+              fastq_qmin, fastq_qmax, fastq_ascii);
+
+      if (fastq_ascii == 64)
         {
-          fprintf(stderr, "     %c %10" PRIu64 " %5.1f%% %6d",
-                  c,
-                  sequence_chars[c],
-                  100.0 * sequence_chars[c] / total_chars,
-                  maxrun[c]);
-          if ((c == 'N') || (c == 'n'))
+          if (qmin < 64)
+            fprintf(stderr, "Guess: Solexa format (phred+64)\n");
+          else if (qmin < 66)
+            fprintf(stderr, "Guess: Illumina 1.3+ format (phred+64)\n");
+          else
+            fprintf(stderr, "Guess: Illumina 1.5+ format (phred+64)\n");
+        }
+      else
+        {
+          if (qmax > 73)
+            fprintf(stderr, "Guess: Illumina 1.8+ format (phred+33)\n");
+          else
+            fprintf(stderr, "Guess: Original Sanger format (phred+33)\n");
+        }
+
+      fprintf(stderr, "\n");
+      fprintf(stderr, "Letter          N   Freq MaxRun\n");
+      fprintf(stderr, "------ ---------- ------ ------\n");
+
+      for(int c=0; c<256; c++)
+        {
+          if (sequence_chars[c] > 0)
             {
-              if (qmin_n < qmax_n)
-                fprintf(stderr, "  Q=%c..%c", qmin_n, qmax_n);
-              else
-                fprintf(stderr, "  Q=%c", qmin_n);
+              fprintf(stderr, "     %c %10" PRIu64 " %5.1f%% %6d",
+                      c,
+                      sequence_chars[c],
+                      100.0 * sequence_chars[c] / total_chars,
+                      maxrun[c]);
+              if ((c == 'N') || (c == 'n'))
+                {
+                  if (qmin_n < qmax_n)
+                    fprintf(stderr, "  Q=%c..%c", qmin_n, qmax_n);
+                  else
+                    fprintf(stderr, "  Q=%c", qmin_n);
+                }
+              fprintf(stderr, "\n");
+            }
+        }
+
+      fprintf(stderr, "\n");
+      fprintf(stderr, "Char  ASCII    Freq       Tails\n");
+      fprintf(stderr, "----  -----  ------  ----------\n");
+
+      for(int c=qmin; c<=qmax; c++)
+        {
+          if (quality_chars[c] > 0)
+            {
+              fprintf(stderr, " '%c'  %5d  %5.1f%%  %10" PRIu64 "\n",
+                      c,
+                      c,
+                      100.0 * quality_chars[c] / total_chars,
+                      tail_chars[c]);
             }
-          fprintf(stderr, "\n");
         }
     }
 
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Char  ASCII    Freq       Tails\n");
-  fprintf(stderr, "----  -----  ------  ----------\n");
-  
-  for(int c=qmin; c<=qmax; c++)
+  if (opt_log)
     {
-      if (quality_chars[c] > 0)
+      fprintf(fp_log, "Read %" PRIu64 " sequences.\n", seq_count);
+
+      fprintf(fp_log, "Qmin %d, QMax %d, Range %d\n",
+              qmin, qmax, qmax-qmin+1);
+
+      fprintf(fp_log, "Guess: -fastq_qmin %d -fastq_qmax %d -fastq_ascii %d\n",
+              fastq_qmin, fastq_qmax, fastq_ascii);
+
+      if (fastq_ascii == 64)
         {
-          fprintf(stderr, " '%c'  %5d  %5.1f%%  %10" PRIu64 "\n",
-                  c,
-                  c,
-                  100.0 * quality_chars[c] / total_chars,
-                  tail_chars[c]);
+          if (qmin < 64)
+            fprintf(fp_log, "Guess: Solexa format (phred+64)\n");
+          else if (qmin < 66)
+            fprintf(fp_log, "Guess: Illumina 1.3+ format (phred+64)\n");
+          else
+            fprintf(fp_log, "Guess: Illumina 1.5+ format (phred+64)\n");
+        }
+      else
+        {
+          if (qmax > 73)
+            fprintf(fp_log, "Guess: Illumina 1.8+ format (phred+33)\n");
+          else
+            fprintf(fp_log, "Guess: Original Sanger format (phred+33)\n");
+        }
+
+      fprintf(fp_log, "\n");
+      fprintf(fp_log, "Letter          N   Freq MaxRun\n");
+      fprintf(fp_log, "------ ---------- ------ ------\n");
+
+      for(int c=0; c<256; c++)
+        {
+          if (sequence_chars[c] > 0)
+            {
+              fprintf(fp_log, "     %c %10" PRIu64 " %5.1f%% %6d",
+                      c,
+                      sequence_chars[c],
+                      100.0 * sequence_chars[c] / total_chars,
+                      maxrun[c]);
+              if ((c == 'N') || (c == 'n'))
+                {
+                  if (qmin_n < qmax_n)
+                    fprintf(fp_log, "  Q=%c..%c", qmin_n, qmax_n);
+                  else
+                    fprintf(fp_log, "  Q=%c", qmin_n);
+                }
+              fprintf(fp_log, "\n");
+            }
+        }
+
+      fprintf(fp_log, "\n");
+      fprintf(fp_log, "Char  ASCII    Freq       Tails\n");
+      fprintf(fp_log, "----  -----  ------  ----------\n");
+
+      for(int c=qmin; c<=qmax; c++)
+        {
+          if (quality_chars[c] > 0)
+            {
+              fprintf(fp_log, " '%c'  %5d  %5.1f%%  %10" PRIu64 "\n",
+                      c,
+                      c,
+                      100.0 * quality_chars[c] / total_chars,
+                      tail_chars[c]);
+            }
         }
     }
 }
@@ -554,7 +612,7 @@ void fastq_stats()
 
   uint64_t filesize = fastq_get_size(h);
 
-  progress_init("Reading fastq file", filesize);
+  progress_init("Reading FASTQ file", filesize);
 
   uint64_t seq_count = 0;
   uint64_t symbols = 0;
@@ -649,10 +707,10 @@ void fastq_stats()
             {
               char * msg;
               if (xsprintf(& msg,
-"FASTQ quality value (%d) out of range (%" PRId64 "-%" PRId64 ").\n"
-"Please adjust the FASTQ quality base character or range with the\n"
-"--fastq_ascii, --fastq_qmin or --fastq_qmax options. For a complete\n"
-"diagnosis with suggested values, please run vsearch --fastq_chars file.",
+                           "FASTQ quality value (%d) out of range (%" PRId64 "-%" PRId64 ").\n"
+                           "Please adjust the FASTQ quality base character or range with the\n"
+                           "--fastq_ascii, --fastq_qmin or --fastq_qmax options. For a complete\n"
+                           "diagnosis with suggested values, please run vsearch --fastq_chars file.",
                            qual, opt_fastq_qmin, opt_fastq_qmax) > 0)
                 fatal(msg);
               else
@@ -735,7 +793,6 @@ void fastq_stats()
 
   if (fp_log)
     {
-      fprintf(fp_log, "\n");
       fprintf(fp_log, "\n");
       fprintf(fp_log, "Read length distribution\n");
       fprintf(fp_log, "      L           N      Pct   AccPct\n");
@@ -802,17 +859,14 @@ void fastq_stats()
       fprintf(fp_log, "    L   1.0000   0.5000   0.2500   0.1000   1.0000   0.5000   0.2500   0.1000\n");
       fprintf(fp_log, "-----  -------  -------  -------  -------  -------  -------  -------  -------\n");
       
-      for(int64_t i = len_max+1; i >= 1; i--)
+      for(int64_t i = len_max; i >= 1; i--)
         {
           int64_t read_count[4];
           double read_percentage[4];
           
           for(int z=0; z<4; z++)
             {
-              if (i>=2)
-                read_count[z] = ee_length_table[4*(i-2)+z];
-              else
-                read_count[z] = seq_count;
+              read_count[z] = ee_length_table[4*(i-1)+z];
               read_percentage[z] = 100.0 * read_count[z] / seq_count;
             }
           
@@ -899,25 +953,29 @@ void fastx_revcomp()
 
   if (opt_fastaout)
     {
-      fp_fastaout = fopen(opt_fastaout, "w");
+      fp_fastaout = fopen_output(opt_fastaout);
       if (!fp_fastaout)
-        fatal("Unable to open fasta output file for writing");
+        fatal("Unable to open FASTA output file for writing");
     }
 
   if (opt_fastqout)
     {
-      fp_fastqout = fopen(opt_fastqout, "w");
+      fp_fastqout = fopen_output(opt_fastqout);
       if (!fp_fastqout)
-        fatal("Unable to open fastq output file for writing");
+        fatal("Unable to open FASTQ output file for writing");
     }
 
   if (h->is_fastq)
-    progress_init("Reading fastq file", filesize);
+    progress_init("Reading FASTQ file", filesize);
   else
-    progress_init("Reading fasta file", filesize);
+    progress_init("Reading FASTA file", filesize);
+
+  int count = 0;
 
   while(fastx_next(h, 0, chrmap_no_change))
     {
+      count++;
+      
       /* header */
       
       uint64_t hlen = fastx_get_header_length(h);
@@ -964,11 +1022,27 @@ void fastx_revcomp()
         }
 
       if (opt_fastaout)
-        fasta_print(fp_fastaout, header, seq_buffer, length);
+        fasta_print_general(fp_fastaout,
+                            0,
+                            seq_buffer,
+                            length,
+                            header,
+                            hlen,
+                            0,
+                            count,
+                            -1, -1, 0, 0.0);
 
       if (opt_fastqout)
-        fastq_print(fp_fastqout, header, seq_buffer, qual_buffer);
-                    
+        fastq_print_general(fp_fastqout,
+                            seq_buffer,
+                            length,
+                            header,
+                            hlen,
+                            qual_buffer,
+                            0,
+                            count,
+                            0, 0.0);
+      
       progress_update(fastx_get_position(h));
     }
   progress_done();
@@ -997,11 +1071,11 @@ void fastq_convert()
 
   FILE * fp_fastqout = 0;
 
-  fp_fastqout = fopen(opt_fastqout, "w");
+  fp_fastqout = fopen_output(opt_fastqout);
   if (!fp_fastqout)
-    fatal("Unable to open fastq output file for writing");
+    fatal("Unable to open FASTQ output file for writing");
 
-  progress_init("Reading fastq file", filesize);
+  progress_init("Reading FASTQ file", filesize);
 
   while(fastq_next(h, 0, chrmap_no_change))
     {
