@@ -75,22 +75,38 @@ void results_show_fastapairs_one(FILE * fp,
                                  hp->nwalignment,
                                  hp->nwalignmentlength,
                                  0);
-      fasta_print_header(fp, query_head);
-      fasta_print_sequence(fp,
-                           qrow + hp->trim_q_left + hp->trim_t_left,
-                           hp->internal_alignmentlength, 0);
+      fasta_print_general(fp,
+                          0,
+                          qrow + hp->trim_q_left + hp->trim_t_left,
+                          hp->internal_alignmentlength,
+                          query_head,
+                          strlen(query_head),
+                          0,
+                          0,
+                          -1,
+                          -1,
+                          0,
+                          0.0);
       xfree(qrow);
-      
+
       char * trow = align_getrow(db_getsequence(hp->target),
                                  hp->nwalignment,
                                  hp->nwalignmentlength,
                                  1);
-      fasta_print_header(fp, db_getheader(hp->target));
-      fasta_print_sequence(fp,
-                           trow + hp->trim_q_left + hp->trim_t_left,
-                           hp->internal_alignmentlength, 0);
+      fasta_print_general(fp,
+                          0,
+                          trow + hp->trim_q_left + hp->trim_t_left,
+                          hp->internal_alignmentlength,
+                          db_getheader(hp->target),
+                          db_getheaderlen(hp->target),
+                          0,
+                          0,
+                          -1,
+                          -1,
+                          0,
+                          0.0);
       xfree(trow);
-      
+
       fprintf(fp, "\n");
     }
 }
@@ -447,9 +463,8 @@ void results_show_alnout(FILE * fp,
           char * dseq = db_getsequence(hp->target);
           int64_t dseqlen = db_getsequencelen(hp->target);
           
-          char dummy;
-          int qlenlen = snprintf(&dummy, 1, "%" PRId64, qseqlen);
-          int tlenlen = snprintf(&dummy, 1, "%" PRId64, dseqlen);
+          int qlenlen = snprintf(0, 0, "%" PRId64, qseqlen);
+          int tlenlen = snprintf(0, 0, "%" PRId64, dseqlen);
           int numwidth = MAX(qlenlen, tlenlen);
           
           fprintf(fp," Query %*" PRId64 "nt >%s\n", numwidth,
