@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2015, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2017, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -63,21 +63,21 @@ void results_show_alnout(FILE * fp,
                          int hitcount,
                          char * query_head,
                          char * qsequence,
-                         long qseqlen,
+                         int64_t qseqlen,
                          char * rc);
 
 void results_show_blast6out_one(FILE * fp,
                                 struct hit * hp,
                                 char * query_head,
                                 char * qsequence,
-                                long qseqlen,
+                                int64_t qseqlen,
                                 char * rc);
 
 void results_show_uc_one(FILE * fp,
                          struct hit * hp,
                          char * query_head,
                          char * qsequence,
-                         long qseqlen,
+                         int64_t qseqlen,
                          char * rc,
                          int clusterno);
 
@@ -85,14 +85,14 @@ void results_show_userout_one(FILE * fp,
                               struct hit * hp,
                               char * query_head,
                               char * qsequence,
-                              long qseqlen,
+                              int64_t qseqlen,
                               char * rc);
 
 void results_show_fastapairs_one(FILE * fp,
                                  struct hit * hp,
                                  char * query_head,
                                  char * qsequence,
-                                 long qseqlen,
+                                 int64_t qseqlen,
                                  char * rc);
 
 void results_show_samheader(FILE * fp,
@@ -104,5 +104,5 @@ void results_show_samout(FILE * fp,
                          int hitcount,
                          char * query_head,
                          char * qsequence,
-                         long qseqlen,
+                         int64_t qseqlen,
                          char * rc);
