@@ -90,6 +90,9 @@ static FILE * fp_otutabout = 0;
 static FILE * fp_mothur_shared_out = 0;
 static FILE * fp_biomout = 0;
 
+static int count_matched = 0;
+static int count_notmatched = 0;
+
 void search_output_results(int hit_count,
                            struct hit * hits,
                            char * query_head,
@@ -205,19 +208,31 @@ void search_output_results(int hit_count,
 
   if (hit_count)
     {
+      count_matched++;
       if (opt_matched)
-        fasta_print(fp_matched,
-                    query_head,
-                    qsequence,
-                    qseqlen);
+        fasta_print_general(fp_matched,
+                            0,
+                            qsequence,
+                            qseqlen,
+                            query_head,
+                            strlen(query_head),
+                            qsize,
+                            count_matched,
+                            -1, -1, 0, 0.0);
     }
   else
     {
+      count_notmatched++;
       if (opt_notmatched)
-        fasta_print(fp_notmatched,
-                    query_head,
-                    qsequence,
-                    qseqlen);
+        fasta_print_general(fp_notmatched,
+                            0,
+                            qsequence,
+                            qseqlen,
+                            query_head,
+                            strlen(query_head),
+                            qsize,
+                            count_notmatched,
+                            -1, -1, 0, 0.0);
     }
 
   /* update matching db sequences */
@@ -460,7 +475,7 @@ void search_prep(char * cmdline, char * progheader)
 
   if (opt_alnout)
     {
-      fp_alnout = fopen(opt_alnout, "w");
+      fp_alnout = fopen_output(opt_alnout);
       if (! fp_alnout)
         fatal("Unable to open alignment output file for writing");
 
@@ -470,89 +485,102 @@ void search_prep(char * cmdline, char * progheader)
 
   if (opt_samout)
     {
-      fp_samout = fopen(opt_samout, "w");
+      fp_samout = fopen_output(opt_samout);
       if (! fp_samout)
         fatal("Unable to open SAM output file for writing");
     }
 
   if (opt_userout)
     {
-      fp_userout = fopen(opt_userout, "w");
+      fp_userout = fopen_output(opt_userout);
       if (! fp_userout)
         fatal("Unable to open user-defined output file for writing");
     }
 
   if (opt_blast6out)
     {
-      fp_blast6out = fopen(opt_blast6out, "w");
+      fp_blast6out = fopen_output(opt_blast6out);
       if (! fp_blast6out)
         fatal("Unable to open blast6-like output file for writing");
     }
 
   if (opt_uc)
     {
-      fp_uc = fopen(opt_uc, "w");
+      fp_uc = fopen_output(opt_uc);
       if (! fp_uc)
         fatal("Unable to open uc output file for writing");
     }
 
   if (opt_fastapairs)
     {
-      fp_fastapairs = fopen(opt_fastapairs, "w");
+      fp_fastapairs = fopen_output(opt_fastapairs);
       if (! fp_fastapairs)
         fatal("Unable to open fastapairs output file for writing");
     }
 
   if (opt_matched)
     {
-      fp_matched = fopen(opt_matched, "w");
+      fp_matched = fopen_output(opt_matched);
       if (! fp_matched)
         fatal("Unable to open matched output file for writing");
     }
 
   if (opt_notmatched)
     {
-      fp_notmatched = fopen(opt_notmatched, "w");
+      fp_notmatched = fopen_output(opt_notmatched);
       if (! fp_notmatched)
         fatal("Unable to open notmatched output file for writing");
     }
 
   if (opt_otutabout)
     {
-      fp_otutabout = fopen(opt_otutabout, "w");
+      fp_otutabout = fopen_output(opt_otutabout);
       if (! fp_otutabout)
         fatal("Unable to open OTU table (text format) output file for writing");
     }
 
   if (opt_mothur_shared_out)
     {
-      fp_mothur_shared_out = fopen(opt_mothur_shared_out, "w");
+      fp_mothur_shared_out = fopen_output(opt_mothur_shared_out);
       if (! fp_mothur_shared_out)
         fatal("Unable to open OTU table (mothur format) output file for writing");
     }
 
   if (opt_biomout)
     {
-      fp_biomout = fopen(opt_biomout, "w");
+      fp_biomout = fopen_output(opt_biomout);
       if (! fp_biomout)
         fatal("Unable to open OTU table (biom 1.0 format) output file for writing");
     }
 
-  db_read(opt_db, 0);
+  /* check if it may be an UDB file */
+
+  bool is_udb = udb_detect_isudb(opt_db);
+
+  if (is_udb)
+    udb_read(opt_db, 1, 1);
+  else
+    db_read(opt_db, 0);
 
   results_show_samheader(fp_samout, cmdline, opt_db);
 
-  if (opt_dbmask == MASK_DUST)
-    dust_all();
-  else if ((opt_dbmask == MASK_SOFT) && (opt_hardmask))
-    hardmask_all();
+  if (!is_udb)
+    {
+      if (opt_dbmask == MASK_DUST)
+        dust_all();
+      else if ((opt_dbmask == MASK_SOFT) && (opt_hardmask))
+        hardmask_all();
+    }
 
   show_rusage();
 
   seqcount = db_getsequencecount();
 
-  dbindex_prepare(1, opt_dbmask);
-  dbindex_addallsequences(opt_dbmask);
+  if (!is_udb)
+    {
+      dbindex_prepare(1, opt_dbmask);
+      dbindex_addallsequences(opt_dbmask);
+    }
 
   /* tophits = the maximum number of hits we need to store */
 
@@ -600,14 +628,14 @@ void usearch_global(char * cmdline, char * progheader)
 
   if (opt_dbmatched)
     {
-      fp_dbmatched = fopen(opt_dbmatched, "w");
+      fp_dbmatched = fopen_output(opt_dbmatched);
       if (! fp_dbmatched)
         fatal("Unable to open dbmatched output file for writing");
     }
 
   if (opt_dbnotmatched)
     {
-      fp_dbnotmatched = fopen(opt_dbnotmatched, "w");
+      fp_dbnotmatched = fopen_output(opt_dbnotmatched);
       if (! fp_dbnotmatched)
         fatal("Unable to open dbnotmatched output file for writing");
     }
@@ -679,23 +707,39 @@ void usearch_global(char * cmdline, char * progheader)
 
   otutable_done();
 
+  int count_dbmatched = 0;
+  int count_dbnotmatched = 0;
+
   if (opt_dbmatched || opt_dbnotmatched)
     {
       for(int64_t i=0; i<seqcount; i++)
         if (dbmatched[i])
           {
+            count_dbmatched++;
             if (opt_dbmatched)
-              {
-                if (opt_sizeout)
-                  fasta_print_db_size(fp_dbmatched, i, dbmatched[i]);
-                else
-                  fasta_print_db(fp_dbmatched, i);
-              }
+              fasta_print_general(fp_dbmatched,
+                                  0,
+                                  db_getsequence(i),
+                                  db_getsequencelen(i),
+                                  db_getheader(i),
+                                  db_getheaderlen(i),
+                                  dbmatched[i],
+                                  count_dbmatched,
+                                  -1, -1, 0, 0.0);
           }
         else
           {
+            count_dbnotmatched++;
             if (opt_dbnotmatched)
-              fasta_print_db(fp_dbnotmatched, i);
+              fasta_print_general(fp_dbnotmatched,
+                                  0,
+                                  db_getsequence(i),
+                                  db_getsequencelen(i),
+                                  db_getheader(i),
+                                  db_getheaderlen(i),
+                                  db_getabundance(i),
+                                  count_dbnotmatched,
+                                  -1, -1, 0, 0.0);
           }
     }
 
