@@ -62,7 +62,7 @@
 
 void shuffle()
 {
-  FILE * fp_output = fopen(opt_output, "w");
+  FILE * fp_output = fopen_output(opt_output);
   if (!fp_output)
     fatal("Unable to open shuffle output file for writing");
 
