@@ -102,7 +102,7 @@ int sortbylength_compare(const void * a, const void * b)
 
 void sortbylength()
 {
-  FILE * fp_output = fopen(opt_output, "w");
+  FILE * fp_output = fopen_output(opt_output);
   if (!fp_output)
     fatal("Unable to open sortbylength output file for writing");
 
